# Calendaring is a simple CMF/Plone calendaring implementation.
# Copyright (C) 2004 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Some portions of this module are Copyright Shuttleworth Foundation.
# The original copyright statement is reproduced below.
#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2003 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
$Id: calendar.py,v 1.4 2005/01/25 01:03:49 dreamcatcher Exp $
"""
import warnings

from Products.CMFCalendar.CalendarTool import calendar
from OFS.PropertyManager import PropertyManager

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo, Unauthorized

from Products.Archetypes.utils import mapply
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.CalendarTool import CalendarTool as BaseTool

class CalendarTool(BaseTool, PropertyManager):
    """ A Calendar Tool implementation
    for import/export of events.
    """
    security = ClassSecurityInfo()

    event_type = 'Event'
    published_states = ('published',)
    _properties = getattr(BaseTool, '_properties', ()) + (
        {'id':'event_type', 'type': 'string', 'mode':'w'},
        {'id':'published_states', 'type': 'lines', 'mode':'w'},)

    manage_options = BaseTool.manage_options + PropertyManager.manage_options

    def __init__(self):
        if hasattr(BaseTool, '__init__'):
            # Plone 2.1 doesn't seem to have a __init__ anymore.
            BaseTool.__init__(self)

    security.declarePrivate('setMapping')
    def setMapping(self, name):
        warnings.warn("portal_calendar.setMapping has been deprecated."
                      "You do not need to call it anymore.",
                      DeprecationWarning, 2)

    security.declarePublic('getDestination')
    def getDestination(self):
        mt = getToolByName(self, 'portal_membership')
        dest = mt.getHomeFolder()
        if dest is None:
            # User does not have a home folder? Try portal and let it fail!
            pt = getToolByName(self, 'portal_url')
            dest = pt.getPortalObject()
        return dest

    security.declarePublic('importCalendar')
    def importCalendar(self, stream, dest=None, kw_map=None, do_action=False):
        from Products.Calendaring.marshaller import CalendarMarshaller
        mt = getToolByName(self, 'portal_membership')
        if mt.isAnonymousUser():
            raise Unauthorized
        if dest is None:
            dest = self.getDestination()
        marshaller = CalendarMarshaller()

        REQUEST = self.REQUEST
        RESPONSE = REQUEST.RESPONSE
        args = [dest, '']
        items = []
        kwargs = {'file':stream,
                  'context':self,
                  'items': items,
                  'REQUEST':REQUEST,
                  'RESPONSE':RESPONSE}
        mapply(marshaller.demarshall, *args, **kwargs)
        return items

    def exportCalendar(self, events=None, REQUEST=None):
        from Products.Calendaring.marshaller import CalendarMarshaller
        marshaller = CalendarMarshaller()

        ddata = marshaller.marshall(self, events=events,
                                    REQUEST=REQUEST,
                                    RESPONSE=None)
        content_type, length, data = ddata

        if REQUEST is not None:
            REQUEST.RESPONSE.setHeader('Content-Type', content_type)
            REQUEST.RESPONSE.setHeader('Content-Length', length)
            REQUEST.RESPONSE.write(data)
        return data

    security.declarePublic('getPublishedStates')
    def getPublishedStates(self):
        return self.getProperty('published_states')

InitializeClass(CalendarTool)



# Calendaring is a simple CMF/Plone calendaring implementation.
# Copyright (C) 2004 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
$Id: __init__.py,v 1.3 2004/09/17 13:58:25 dreamcatcher Exp $
"""

import os
import sys
from Globals import package_home
from Products.Archetypes.public import *
from Products.CMFCore import utils as cmf_utils
from Products.CMFPlone import utils as plone_utils

from Products.Calendaring.config import *
from Products.Calendaring.marshaller import CalendarMarshaller
from Products.Calendaring.marshaller import EventMarshaller

# Kickstart Extensions.
from Products.Calendaring.Extensions import Install as _Install
del _Install

# If Marshall is available, register the marshallers.
try:
    from Products.Marshall.registry import registerComponent
    registerComponent('calendaring_calendar', 'Calendar Marshaller',
                      CalendarMarshaller)
    registerComponent('calendaring_event', 'Event Marshaller',
                      EventMarshaller)
except ImportError:
    pass

def initialize(context):
    from Products.Calendaring.tools import calendar
    from Products.Calendaring.common import DT2dt

    try:
        from Products.ATContentTypes.content import event
        event.DT2dt = DT2dt
        log('Patching ATContentTypes to return UTC datetime.')
    except ImportError:
        pass

    tools = (calendar.CalendarTool,)

    plone_utils.ToolInit('%s Tools' % PROJECTNAME, 
                         tools=tools, 
                         icon='tool.gif',
                         ).initialize(context)
    del calendar

    from Products.Calendaring.content import calendar

    # register archetypes content with the machinery
    content_types, constructors, ftis = process_types(listTypes(PROJECTNAME),
                                                      PROJECTNAME)
    cmf_utils.ContentInit(
        PROJECTNAME + ' Content',
        content_types = content_types,
        permission = ADD_CONTENT_PERMISSION,
        extra_constructors = constructors,
        fti = ftis).initialize(context)



# Calendaring is a simple CMF/Plone calendaring implementation.
# Copyright (C) 2004 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
$Id: test_content.py,v 1.8 2005/01/25 01:03:48 dreamcatcher Exp $
"""


import os, sys
from os import curdir
from os.path import join, abspath, dirname, split
from StringIO import StringIO
from dateutil.tz import tzutc

try:
    __file__
except NameError:
    # Test was called directly, so no __file__ global exists.
    _prefix = abspath(curdir)
else:
    # Test was called by another test.
    _prefix = abspath(dirname(__file__))

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

# Load fixture
import unittest
from Testing import ZopeTestCase
from Testing.ZopeTestCase import doctest
from Testing.ZopeTestCase import FunctionalDocFileSuite as DocSuite

# Install our product
ZopeTestCase.installProduct('MimetypesRegistry')
ZopeTestCase.installProduct('PortalTransforms')
ZopeTestCase.installProduct('Archetypes')
ZopeTestCase.installProduct('Calendaring')
ZopeTestCase.installProduct('ATContentTypes')

import datetime
from DateTime import DateTime
from Products.CMFPlone.tests import PloneTestCase
from Products.CMFCore.tests.base.utils import has_path
from Products.Archetypes.utils import mapply
from Products.Calendaring.tools import calendar
from Products.Calendaring.Extensions.Install import install_tools
from Products.Calendaring.tests.utils import installType
from Products.Calendaring.common import toTime
from Products.Calendaring.marshaller import CalendarMarshaller

def fromFile(obj, stream):
    REQUEST = obj.REQUEST
    RESPONSE = REQUEST.RESPONSE
    marshaller = CalendarMarshaller()
    args = [obj, '']
    items = []
    kwargs = {'file':stream,
              'context':obj,
              'items': items,
              'REQUEST':REQUEST,
              'RESPONSE':RESPONSE}
    mapply(marshaller.demarshall, *args, **kwargs)
    return items

class TestImportCalendar(PloneTestCase.PloneTestCase):

    def _setupCalendar(self):
        self.qi = self.portal.portal_quickinstaller
        # Replace tool manually, to test normal events
        install_tools(self.portal, 'Calendaring',
                      [calendar.CalendarTool.meta_type], StringIO())
        self.cal = self.portal.portal_calendar

        # Install CMF Event as Event.
        pt = self.portal.portal_types
        if 'Event' in pt.objectIds():
            pt.manage_delObjects(ids=['Event'])
        pt.manage_addTypeInformation(
            id='Event',
            add_meta_type='Factory-based Type Information',
            typeinfo_name='CMFCalendar: Event (CMF Event)')
        pt._getOb('Event').manage_changeProperties(
            product='CMFCalendar',
            factory='addEvent',
            title='Event')

    def afterSetUp(self):
        self.loginAsPortalOwner()
        self._setupCalendar()
        self.ct = self.portal.portal_catalog
        self.mt = self.portal.portal_membership
        self.wt = self.portal.portal_workflow
        self.mt.createMemberarea()

    def testImport1(self):
        self.file = open(join(_prefix, 'data', 'example.ics'), 'r')
        self.cal.importCalendar(self.file, do_action=True)
        self.file.close()
        result = self.cal.catalog_getevents(year=2002, month=10)
        events = result[28]['eventslist']
        self.assertEquals(len(events), 1)
        # US/Pacific = GMT-8
        self.assertEquals(events[0]['start'], '19:00:00')
        self.assertEquals(events[0]['end'], '20:00:00')
        self.assertEquals(events[0]['title'], 'Coffee with Jason')
        result = self.cal.catalog_getevents(year=2002, month=11)
        events = result[27]['eventslist']
        self.assertEquals(len(events), 1)
        self.assertEquals(events[0]['start'], '18:00:00')
        self.assertEquals(events[0]['end'], '19:00:00')
        self.assertEquals(events[0]['title'], 'Code Review')
        result = self.cal.catalog_getevents(year=2002, month=12)
        events = result[17]['eventslist']
        self.assertEquals(len(events), 1)
        self.assertEquals(events[0]['start'], '02:00:00')
        self.assertEquals(events[0]['end'], '03:00:00')
        self.assertEquals(events[0]['title'], 'Dinner with T')
        gef = self.cal.getEventsForThisDay
        self.assertEquals(len(gef(DateTime(2002, 10, 28))), 1)
        self.assertEquals(len(gef(DateTime(2002, 11, 27))), 1)
        self.assertEquals(len(gef(DateTime(2002, 12, 17))), 1)
        # Test timezone naive allday event:
        result = self.cal.catalog_getevents(year=2002, month=12)
        events = result[15]['eventslist']
        self.assertEquals(len(events), 1)
        self.assertEquals(events[0]['start'], '00:00:00')
        # Yes, it returns None for all day events
        self.assertEquals(events[0]['end'], None)
        self.assertEquals(events[0]['title'], 'Timezone naive event')

    def testImport2(self):
        self.file = open(join(_prefix, 'data', 'tasks.ics'), 'r')
        self.items = self.cal.importCalendar(self.file, do_action=True)
        self.file.close()
        result = self.cal.catalog_getevents(year=2004, month=4)
        events = result[26]['eventslist']
        self.assertEquals(len(events), 1)
        self.assertEquals(events[0]['start'], '15:30:00')
        self.assertEquals(events[0]['end'], '22:30:00')
        self.assertEquals(events[0]['title'], 'Formal Languages Exam')
        item = self.items[0][1]

        # toTime gives UTC time, date is in GMT-3
        self.assertEquals(toTime(item.start_date), '12:00:00')
        self.assertEquals(toTime(item.end_date), '18:00:00')
        self.assertEquals(item.Title(), 'Send Invoice')
        self.assertEquals(self.wt.getInfoFor(item, 'review_state'), 'private')
        gef = self.cal.getEventsForThisDay
        self.assertEquals(len(gef(DateTime(2004, 4, 26))), 1)

    def testImport3(self):
        self.file = open(join(_prefix, 'data', 'frisco.ics'), 'r')
        self.items = self.cal.importCalendar(self.file, do_action=True)
        self.file.close()
        result = self.cal.catalog_getevents(year=2004, month=12)
        events = result[6]['eventslist']
        self.assertEquals(len(events), 1)

        # US/Eastern = GMT-6
        self.assertEquals(events[0]['start'], '23:30:00')
        self.assertEquals(events[0]['end'], None)
        self.assertEquals(events[0]['title'], 'Brazilian jazz at Berkeley')

        events = result[7]['eventslist']
        self.assertEquals(len(events), 1)
        self.assertEquals(events[0]['start'], None)
        self.assertEquals(events[0]['end'], '00:30:00')
        self.assertEquals(events[0]['title'], 'Brazilian jazz at Berkeley')

        item = self.items[0][1]
        self.assertEquals(toTime(item.start_date), '01:30:00')
        self.assertEquals(toTime(item.end_date), '02:30:00')
        self.assertEquals(item.Title(), 'Brazilian jazz at Berkeley')
        self.assertEquals(self.wt.getInfoFor(item, 'review_state'), 'published')
        gef = self.cal.getEventsForThisDay
        self.assertEquals(len(gef(DateTime(2004, 12, 6))), 1)
        self.assertEquals(len(gef(DateTime(2004, 12, 7))), 0)

    def testImport4(self):
        self.portal.invokeFactory('Folder', id='test_calendar')
        cal = self.portal.test_calendar
        self.file = open(join(_prefix, 'data', 'no_dtend.ics'), 'r')
        self.items = fromFile(cal, self.file)
        self.file.close()

        item = self.items[0][1]
        self.failUnlessEqual(item.start_date, item.end_date)

class TestContentTypes(TestImportCalendar):
    # This test uses the default Event type in Plone (ATEvent, from
    # ATContentTypes at the time of this writing), and expects it to
    # be working and stable.

    def _setupCalendar(self):
        self.qi = self.portal.portal_quickinstaller
        self.qi.installProduct('PortalTransforms')
        self.qi.installProduct('Archetypes')
        self.qi.installProduct('Calendaring')
        self.cal = self.portal.portal_calendar

    def _checkTasks(self, cal, result, items):
        events = result[26]['eventslist']
        self.assertEquals(len(events), 1, events)
        self.assertEquals(events[0]['start'], '15:30:00')
        self.assertEquals(events[0]['end'], '22:30:00')
        self.assertEquals(events[0]['title'], 'Formal Languages Exam')

        item = items[0][1]

        # toTime returns UTC time.
        self.assertEquals(toTime(item.start_date), '12:00:00')
        self.assertEquals(toTime(item.end_date), '18:00:00')
        self.assertEquals(item.Title(), 'Send Invoice')
        self.assertEquals(self.wt.getInfoFor(item, 'review_state'), 'private')

        gef = self.cal.getEventsForThisDay
        self.assertEquals(len(gef(DateTime(2004, 4, 26))), 1)
        self.wt.doActionFor(item, 'show')
        self.assertEquals(len(gef(DateTime(2004, 4, 26))), 1)
        self.wt.doActionFor(item, 'submit')
        self.assertEquals(len(gef(DateTime(2004, 4, 26))), 1)
        self.wt.doActionFor(item, 'publish')
        self.assertEquals(len(gef(DateTime(2004, 4, 26))), 2)
        self.wt.doActionFor(item, 'retract')
        self.assertEquals(len(gef(DateTime(2004, 4, 26))), 1)
        self.wt.doActionFor(item, 'hide')
        self.assertEquals(len(gef(DateTime(2004, 4, 26))), 1)

        item = items[1][1]

        # toTime returns UTC time.
        self.assertEquals(toTime(item.start_date), '18:30:00')
        self.assertEquals(toTime(item.end_date), '01:30:00')
        self.assertEquals(item.Title(), 'Formal Languages Exam')
        self.assertEquals(self.wt.getInfoFor(item, 'review_state'), 'published')


    def testCalendar(self):
        self.portal.invokeFactory('Folder', id='test_calendar')
        cal = self.portal.test_calendar
        self.portal.invokeFactory('Folder', id='test_calendar2')
        cal2 = self.portal.test_calendar2
        self.file = open(join(_prefix, 'data', 'tasks.ics'), 'r')
        self.items = fromFile(cal, self.file)

        self.assertEquals(len(cal.objectIds()), 2)
        self.assertEquals(len(cal2.objectIds()), 0)

        result = self.cal.catalog_getevents(year=2004, month=4)
        self._checkTasks(cal, result, self.items)

        self.file.seek(0)
        fromFile(cal2, self.file)
        self.file.close()

        self.assertEquals(len(cal2.objectIds()), 2)

    def testKOrganizer(self):
        self.portal.invokeFactory('Folder', id='test_calendar')
        cal = self.portal.test_calendar
        self.file = open(join(_prefix, 'data', 'korganizer.ics'), 'r')
        self.items = fromFile(cal, self.file)
        self.file.close()

        item = self.items[0][1]
        value = '"Sidnei da Silva" <sidnei@awkly.org>'
        attendees = item.getAttendees()
        self.failUnless(value in attendees, (value, attendees))
        self.assertEquals(len(item.getAttendees()), 3, item.attendees)
        self.assertEquals(item.getLocation(), 'Plone.org')

        item = self.items[1][1]
        self.assertEquals(len(item.getAttendees()), 0)
        self.assertEquals(item.getLocation(), 'University')

        item = self.items[2][1]
        self.assertEquals(len(item.getAttendees()), 0)
        self.assertEquals(item.getLocation(), "Mom's House")

        item = self.items[3][1]
        attendees = item.getAttendees()
        value = '"Sidnei da Silva" <sidnei@awkly.org>'
        self.failUnless(value in attendees, (value, attendees))
        value = 'Kitchen Assembler'
        self.failUnless(value in attendees, (value, attendees))
        self.assertEquals(len(item.getAttendees()), 2)
        self.assertEquals(item.getLocation(), 'My Apartment')
        self.assertEquals(item.Title(), 'Assemble Kitchen')
        self.assertEquals(item.Description(),
                          ('Help and supervise the kitchen '
                           'assembling guy. All day :('))

        item = self.items[4][1]
        attendees = item.getAttendees()
        value = '"Sidnei da Silva" <sidnei@awkly.org>'
        self.failUnless(value in attendees, (value, attendees))
        value = '"Paul Everitt" <paul@zope-europe.org>'
        self.failUnless(value in attendees, (value, attendees))
        self.assertEquals(len(item.getAttendees()), 2)
        self.assertEquals(item.getLocation(), 'Europe')
        self.assertEquals(item.Title(), 'Send Invoice')
        self.assertEquals(item.Description(),
                          'Bill for 10h sysadm + 20h development\n')

    def testOutlook1(self):
        self.portal.invokeFactory('Folder', id='test_calendar')
        cal = self.portal.test_calendar
        self.file = open(join(_prefix, 'data', 'outlook1.ics'), 'r')
        self.items = fromFile(cal, self.file)
        self.file.close()

        item = self.items[0][1]
        self.failIf(item.getAttendees())
        self.assertEquals(item.getLocation(), 'Anywhere')
        self.assertEquals(item.Title(), 'Send event to runyaga')
        self.assertEquals(toTime(item.start_date), '08:00:00')
        self.assertEquals(toTime(item.end_date), '08:30:00')

    def testOutlook2(self):
        self.portal.invokeFactory('Folder', id='test_calendar')
        cal = self.portal.test_calendar
        self.file = open(join(_prefix, 'data', 'outlook2.ics'), 'r')
        self.items = fromFile(cal, self.file)
        self.file.close()

        item = self.items[0][1]
        self.failIf(item.getAttendees())
        self.assertEquals(item.Title(), 'My Subject')
        self.assertEquals(item.getLocation(), 'My Location')
        self.assertEquals(item.Description(), 'This is a note.\n')
        self.assertEquals(toTime(item.start_date), '00:00:00')
        self.assertEquals(toTime(item.end_date), '00:00:00')
        self.assertEquals(item.start_date,
                          datetime.datetime(2004, 4, 27, tzinfo=tzutc()))
        self.assertEquals(item.end_date,
                          datetime.datetime(2004, 4, 28, tzinfo=tzutc()))
        self.assertEquals(tuple(item.Subject()), ('Webrent2',))

    def testOutlook3(self):
        self.portal.invokeFactory('Folder', id='test_calendar')
        cal = self.portal.test_calendar
        self.file = open(join(_prefix, 'data', 'outlook3.ics'), 'r')
        self.items = fromFile(cal, self.file)
        self.file.close()

        item = self.items[0][1]
        attendees = item.getAttendees()
        self.failUnless(attendees)
        values = (
            '"Sidnei da Silva" <sidnei@enfoldsystems.com>',
            '"George Alan Runyan" <alan@enfoldsystems.com>',
            '"Toby Roberts" <toby@enfoldsystems.com>',
            '"Andy McKay" <andy@enfoldsystems.com>')
        for value in values:
            self.failUnless(value in attendees, (value, attendees))

        self.assertEquals(item.Title(), 'Test Event: Weekly Meeting')
        self.assertEquals(item.getLocation(), 'Enfold Systems')
        self.assertEquals(item.Description().strip(), '')
        self.assertEquals(toTime(item.start_date), '15:30:00')
        self.assertEquals(toTime(item.end_date), '16:00:00')
        self.assertEquals(item.start_date,
                          datetime.datetime(2005, 10, 21, 15, 30, tzinfo=tzutc()))
        self.assertEquals(item.end_date,
                          datetime.datetime(2005, 10, 21, 16, 0, tzinfo=tzutc()))
        self.assertEquals(tuple(item.Subject()), ())

    def testSunbird(self):
        self.portal.invokeFactory('Folder', id='test_calendar')
        cal = self.portal.test_calendar
        self.file = open(join(_prefix, 'data', 'sunbird.ics'), 'r')
        self.items = fromFile(cal, self.file)
        self.file.close()

        item = self.items[0][1]
        self.failIf(item.getAttendees())
        self.failIf(item.getLocation())
        self.assertEquals(item.Title(), 'IIS Caching')
        # toTime gives UTC, dates are interpreted as local.
        self.assertEquals(toTime(item.start_date), '19:00:00')
        self.assertEquals(toTime(item.end_date), '19:00:00')

    def testiCal(self):
        self.portal.invokeFactory('Folder', id='test_calendar')
        cal = self.portal.test_calendar
        self.file = open(join(_prefix, 'data', 'ical.ics'), 'r')
        self.items = fromFile(cal, self.file)
        self.file.close()

        values = (
            'Tev rehearsal', 'Gig with Tev',
            'BJS Jam Sessions', 'Reason class',
            'Soulive', 'Dirty Dozen Brass Band',
            'Thaddeus Hogarth', 'Anthony Braxton Sextet',
            'BJS Meeting', 'Tev rehearsal', 'Tev',
            'Boston Jazz Now: A Look at the Current Trends on the Boston Scene'
            )
        for value, item in zip(values, self.items):
            self.assertEquals(item[1].Title(), value)

        self.assertEquals(
            self.items[-3][1].Description(),
            '\n'.join((
            "94 1/2 Gore St. #3",
            "T to Lechmere. Get off at Gov't ctr and take shuttle",
            "Walk down Cambridge ", # Note the extra space.
            "R on Sciarratta",
            "L on Gore",
            "top buzzer")))

        # iCal seems to think that double quotes need escaping. That
        # is wrong according to the spec, so we manually fix it up in
        # the marshaller.
        self.assertEquals(
            self.items[-6][1].Description(),
            ('"Take a healthy dose of Sly and The Family Stone, '
             'add a liberal pinch of Stevie Wonder, put on your '
             'dancing shoes and boogie to the music of Thaddeus '
             'Hogarth" - Bruce Gelerman, NPR.'))


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(doctest.DocTestSuite(
        'Products.Calendaring.common',
        optionflags=doctest.ELLIPSIS | doctest.REPORT_UDIFF))
    suite.addTest(doctest.DocTestSuite(
        'Products.Calendaring.marshaller',
        optionflags=doctest.ELLIPSIS | doctest.REPORT_UDIFF))
    suite.addTest(unittest.makeSuite(TestContentTypes))
    suite.addTest(unittest.makeSuite(TestImportCalendar))
    return suite

if __name__ == '__main__':
    framework(descriptions=1, verbosity=1)



from zope.interface import implements
from zope.component.interfaces import IObjectEvent
from zope.component.interfaces import ObjectEvent

class IDemarshalledVEventEvent(IObjectEvent):
    """An event indicating that a VEvent was demarshalled"""

class DemarshalledVEventEvent(ObjectEvent):
    implements(IDemarshalledVEventEvent)



# Calendaring is a simple CMF/Plone calendaring implementation.
# Copyright (C) 2004 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
$Id: marshaller.py 59242 2008-02-23 02:17:57Z dreamcatcher $
"""

import re
import transaction

from urllib import quote
from types import ListType, TupleType
from datetime import datetime, date
from dateutil.tz import gettz, tzutc
from rfc822 import parseaddr, dump_address_pair
from icalendar import Calendar as iCalendar, Event as VEvent
from icalendar.prop import vCalAddress, vDate
from icalendar.prop import vDatetime, vText, vDDDTypes
from icalendar.parser import Parameters
import zope.event

from StringIO import StringIO
from Acquisition import aq_base
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.WorkflowCore import WorkflowException
from Products.CMFCalendar.Event import Event as CMFEvent
from Products.Archetypes.Marshall import Marshaller

from Products.Calendaring.config import log
from Products.Calendaring.common import dt2DT, utc_strftime
from Products.Calendaring.zopeevents import DemarshalledVEventEvent

ICAL_FIXUP = re.compile(r'\\"')
UID_ATTR = '_event_uid'

def generateUID(event):
    uid = getattr(aq_base(event), UID_ATTR, None)
    if uid is not None:
        return uid
    id = event.getId()
    if id:
        return id
    h_event = abs(hash(repr(event)))
    h_id = abs(hash(id))
    return "%s-%s-RID" % (h_event, h_id)

def _cleanId(id):
    quoted = quote(id.encode('utf-8'))
    return quoted.replace('-', '--').replace('%', '-')

def _generateId(uid, title, context=None):
    pt = getToolByName(context, 'plone_utils', None)
    if uid:
        if pt is not None:
            return pt.normalizeString(uid)
        else:
            return _cleanId(uid)
    if pt is not None:
        return pt.normalizeString(title)
    return _cleanId(id)

def generateId(event, context=None):
    uid = event.decoded('uid', None)
    title = event.decoded('summary', None)
    return _generateId(uid, title, context)

def person2text(value):
    name = value.params.get('cn', '')
    email = value.decode().lower()
    if not 'mailto:' in email:
        email = ''
    else:
        email = filter(None, email.split('mailto:'))
        email = email and email[0] or ''
    if email:
        return dump_address_pair((name, email))
    return name

def prop2Text(prop):
    """Convert a vText instance to a string.

    Fixup iCal, which thinks double quotes need to be escaped.

    >>> a = vText('\"The brown fox jumps over the lazy dog\" -- said Sidnei')
    >>> prop2Text(a)
    u'"The brown fox jumps over the lazy dog" -- said Sidnei'

    """
    return ICAL_FIXUP.sub('"', prop)

def prop2DateTime(prop):
    """Convert a vDDDTypes instance with TZID in params to a DateTime in UTC.

    Should be 10am BRT, converted to 10am local (GMT-2).

    >>> import os
    >>> os.environ['TZ'] = 'Brazil/East'

    >>> a = vDDDTypes(datetime(2005, 11, 10, 10, 0, 0))
    >>> a.params = Parameters({'tzid': 'Brazil/East'})
    >>> prop2DateTime(a)
    DateTime('2005/11/10 10:00:00 GMT-2')

    Should be 10am BRT, converted to 10am local (GMT-3).

    >>> a = vDDDTypes(datetime(2005, 07, 10, 10, 0, 0))
    >>> a.params = Parameters({'tzid': 'Brazil/East'})
    >>> prop2DateTime(a)
    DateTime('2005/07/10 10:00:00 GMT-3')

    Should be 10am UTC, converted to 8am local.

    >>> a = vDDDTypes(datetime(2005, 11, 10, 10, 0, 0, tzinfo=tzutc()))
    >>> prop2DateTime(a)
    DateTime('2005/11/10 08:00:00 GMT-2')

    Should be 10am local time, converted to 10am local.
    >>> a = vDDDTypes(datetime(2005, 11, 10, 10, 0, 0))
    >>> prop2DateTime(a)
    DateTime('2005/11/10 10:00:00 GMT-2')
    """
    dt = prop.dt
    tzid = None
    if isinstance(dt, datetime):
        params = getattr(prop, 'params', {})
        tzid = params.get('tzid', None)
    return dt2DT(dt, tzid, local=True)

def convert(value, encoding='utf-8'):
    def _convert(value, encoding=encoding):
        for t, c in converters.items():
            if isinstance(value, t):
                value = c(value)
                break
        #if hasattr(value, 'decode'):
            #value = value.decode()
        if isinstance(value, unicode):
            value = value.encode(encoding)
        return value
    if type(value) not in (ListType, TupleType):
        return _convert(value)
    return map(_convert, value)


status_mapping = {'TENTATIVE':('submit', 'reject', 'retract', 'show'),
                  'CONFIRMED':('publish',),
                  'CANCELLED':('hide',),
                  }
re_status_mapping = {'published':'CONFIRMED',
                     'pending':'TENTATIVE',
                     'visible':'TENTATIVE',
                     'private':'CANCELLED'}

converters = {datetime : dt2DT,
              date : dt2DT,
              vText: prop2Text,
              vDDDTypes: prop2DateTime,
              vCalAddress: person2text}

kw_mappings = []
default_mapping = {'dtstart':'startDate',
                   'dtend':'endDate',
                   'summary':'title',
                   'description':'description',
                   'location':'location',
                   'attendee':'attendees',
                   'categories':'eventType',
                   'url':'eventUrl'}

# The following is to support CMF Event.
kw_mappings.append((CMFEvent, {'dtstart':'start_date',
                               'dtend':'end_date',
                               'summary':'title',
                               'description':'description'}))


cmf_update = {'start_date': 'setStartDate',
              'end_date': 'setEndDate',
              'title': 'setTitle',
              'description': 'setDescription',
              'url': 'setEventURL'}

def cmf_event_update(self, **kw):
    for key, name in cmf_update.items():
        value = kw.get(key)
        if value:
            method = getattr(self, name)
            method(value)
    self.reindexObject()

def cmf_set_event_url(self, event_url):
    self.event_url = event_url

CMFEvent.update = cmf_event_update
CMFEvent.setEventURL = cmf_set_event_url

def set_event_info_from_vevent(instance, ev, kw_map, do_action=True):
    # Make sure a subtransaction happens. We might end up renaming the
    # event, and we don't really know if it was existing or new.
    transaction.savepoint(optimistic=True)
    info = {}
    for k, v in kw_map.items():
        value = ev.get(k, None)
        if value is None:
            # Ugh, special case for dtend
            if k in ('dtend',):
                # dtstart and duration *must* exist.
                dtstart = ev.get('dtstart')
                params = getattr(dtstart, 'params', {})
                duration = ev.get('duration')
                if duration is None:
                    dtend = dtstart.dt
                else:
                    dtend = dtstart.dt + duration.dt
                value = vDDDTypes(dtend)
                value.params = Parameters(dict(params))
            else:
                continue
        value = convert(value)
        info[v] = value

    instance.update(**info)
    uid = ev.decoded('uid', None)
    if uid is None:
        uid = generateUID(instance)
    setattr(instance, UID_ATTR, uid)

    if do_action:
        # XXX NAAAAASTY Code. We need to create a special
        # workflow for Event. This assumes the default Plone workflow.
        wf = getToolByName(instance, 'portal_workflow')
        state = wf.getInfoFor(instance, 'review_state')
        status = ev.get('status', 'CONFIRMED')
        if not re_status_mapping.get(state, 'CONFIRMED') == status:
            set_event_state_from_status(instance, status, wf)

    instance.reindexObject()

def set_event_state_from_status(instance, status, wf=None):
    if wf is None:
        wf = getToolByName(instance, 'portal_workflow')
    actions = status_mapping.get(status, 'publish')
    for action in actions:
        try:
            wf.doActionFor(instance, action)
        except WorkflowException:
            # Try to revert to visible
            for a in ('retract', 'show'):
                try:
                    wf.doActionFor(instance, 'retract')
                except WorkflowException:
                    pass
            try:
                wf.doActionFor(instance, action)
            except WorkflowException:
                pass
            else:
                break
        else:
            break
    state = wf.getInfoFor(instance, 'review_state')
    assert re_status_mapping.get(state, 'CONFIRMED') == status

def marshall_event_to_vevent(instance):
    wf = getToolByName(instance, 'portal_workflow')
    dtstamp = utc_strftime(datetime.utcnow(), "%Y%m%dT%H%M%SZ")
    uid_hash = abs(hash(instance))
    state = wf.getInfoFor(instance, 'review_state')
    status = re_status_mapping.get(state, 'TENTATIVE')
    event = VEvent()
    event.add('uid', generateUID(instance))
    event.add('summary', instance.Title(), encode=False)
    event.add('description', instance.Description(), encode=False)
    if hasattr(aq_base(instance), 'getLocation'):
        location = instance.getLocation()
        if location:
            event.add('location', location, encode=False)
    if hasattr(aq_base(instance), 'Subject'):
        subject = instance.Subject()
        if subject:
            for category in subject:
                event.add('categories', category, encode=False)
    if hasattr(aq_base(instance), 'event_url'):
        url = instance.event_url
        if callable(url):
            # Ugh, it's callable in ATContentTypes.
            url = url()
        if url:
            event.add('url', url)
            event['url'].params['value'] = 'URI'
    if hasattr(aq_base(instance), 'getAttendees'):
        for att in instance.getAttendees():
            name, email = parseaddr(att)
            if not name:
                name = att
                email = ''
            if email:
                email = 'mailto:%s' % email
            else:
                email = 'invalid:nomail'
            address = vCalAddress(email)
            address.params['cn'] = name
            event.add('attendee', address, encode=False)
    if (hasattr(aq_base(instance), 'duration') and
        instance.duration == date.resolution):
        # All day event.
        dtstart = vDate.from_ical(
            utc_strftime(instance.start_date, '%Y%m%d'))
        event.add('dtstart', dtstart)
        dtend = vDate.from_ical(
            utc_strftime(instance.end_date, '%Y%m%d'))
        event.add('dtend', dtend)
    else:
        # Normal event
        dtstart = vDatetime.from_ical(
            utc_strftime(instance.start_date, '%Y%m%dT%H%M%SZ'))
        event.add('dtstart', dtstart)
        dtend = vDatetime.from_ical(
            utc_strftime(instance.end_date, '%Y%m%dT%H%M%SZ'))
        event.add('dtend', dtend)
    dtstamp = vDatetime.from_ical(dtstamp)
    event.add('dtstamp', dtstamp)
    event.add('status', status)
    return event

def wrap_into_vcalendar(*vevents, **kw):
    calendar = iCalendar()
    calendar.add('prodid',  "-//Plone/NONSGML Calendaring//EN")
    calendar.add('version', '2.0')
    title = kw.get('title')
    if title:
        title = unicode(title, 'utf-8')
        calendar.add('x-wr-calname', title)
    for vevent in vevents:
        calendar.add_component(vevent)
    return calendar.as_string()

class EventMarshaller(Marshaller):

    def marshall(self, instance, **kw):
        vevent = marshall_event_to_vevent(instance)
        vcal = wrap_into_vcalendar(vevent)
        return ('text/calendar', len(vcal), vcal)

    def demarshall(self, instance, data, **kw):
        file = kw.get('file', None)
        if file is not None:
            if not data:
                data = file.read()
        calendars = iCalendar.from_string(data, multiple=True)
        do_action = kw.get('do_action', True)
        kw_map = default_mapping
        for klass, mapping in kw_mappings:
            if isinstance(instance, klass):
                kw_map = mapping
        for calendar in calendars:
            for ev in calendar.walk('VEVENT'):
                # We only care about the first one here.
                set_event_info_from_vevent(instance, ev, kw_map,
                                           do_action=do_action)
                break

class CalendarMarshaller(Marshaller):

    def marshall(self, instance, **kw):
        events = kw.get('events', None)
        if events is None:
            event_types = getattr(instance, 'event_types', ('ATEvent', 'Event'))
            events = instance.contentValues(
                filter={'portal_type':event_types})
        events.sort()
        vevents = [marshall_event_to_vevent(event) for event in events]
        if not vevents:
            # There were no events.  iCalendar spec (RFC 2445) requires
            # VCALENDAR to have at least one subcomponent.  Let's create
            # a fake event.
            # NB Mozilla Calendar produces a 0-length file when publishing
            # empty calendars.  Sadly it does not then accept them
            # (http://bugzilla.mozilla.org/show_bug.cgi?id=229266).
            dtstamp = datetime.utcnow().strftime("%Y%m%dT%H%M%SZ")
            event = VEvent()
            vevents.append(event)
            event.add('uid', "placeholder-%s" % dtstamp)
            event.add('summary', 'Empty calendar')
            dtstart = vDatetime.from_ical(dtstamp[:-1])
            event.add('dtstart', dtstart)
            dtstamp = vDatetime.from_ical(dtstamp)
            event.add('dtstamp', dtstamp)
        vcal = wrap_into_vcalendar(*vevents, **{'title': instance.Title()})
        return ('text/calendar', len(vcal), vcal)

    def demarshall(self, instance, data, **kw):
        file = kw.get('file', None)
        if file is not None:
            if not data:
                data = file.read()
        calendars = iCalendar.from_string(data, multiple=True)
        do_action = kw.get('do_action', True)
        items = kw.get('items', [])

        title = None
        for calendar in calendars:
            if title is None:
                for cal in calendar.walk('VCALENDAR'):
                    title = cal.get('x-wr-calname', None) # Apple's iCal Title
                    if title is not None:
                        title = convert(title)
                        break
            for ev in calendar.walk('VEVENT'):
                # We figure out the mapping after the first one has been
                # created, or at least one exists.
                id = generateId(ev, context=instance)
                event = instance._getOb(id, default=None)
                created = False
                if event is None:
                    instance.invokeFactory('Event', id)
                    event = instance._getOb(id)
                    created = True
                items.append((id, event))
                # Detect kw_map to use. Can be different for different
                # kinds of events in the same calendar. That's a fairly
                # cornerish case though.
                kw_map = None
                for klass, mapping in kw_mappings:
                    if isinstance(event, klass):
                        kw_map = mapping
                if kw_map is None:
                    kw_map = default_mapping
                set_event_info_from_vevent(event, ev, kw_map,
                                           do_action=do_action)

                if created:
                    zope.event.notify(DemarshalledVEventEvent(event))

        if title is not None:
            if hasattr(aq_base(instance), 'setTitle'):
                instance.setTitle(title)



# Calendaring is a simple CMF/Plone calendaring implementation.
# Copyright (C) 2004 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
$Id: calendar.py,v 1.8 2005/01/11 00:26:34 dreamcatcher Exp $
"""
from types import MethodType, FunctionType
from Acquisition import aq_base
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from zExceptions import MethodNotAllowed, NotFound, Unauthorized

from Products.Archetypes.public import *
from Products.Archetypes.utils import mapply
from Products.Calendaring.config import *
from Products.Calendaring.marshaller import CalendarMarshaller
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.permissions import ModifyPortalContent, View

calendar_marshaller = CalendarMarshaller()
try:
    from Products.Marshall import ControlledMarshaller
    calendar_marshaller = ControlledMarshaller(calendar_marshaller)
except ImportError:
    pass

class Calendar(BaseFolder):

    meta_type = portal_type = archetype_name = 'Calendar'
    filter_content_types = 1
    event_types = ('ATEvent', 'Event')
    allowed_content_types = event_types + ('Calendar', )
    _at_rename_after_creation = True
    schema = (BaseFolderSchema +
              Schema((), marshall=calendar_marshaller))

    security = ClassSecurityInfo()

    __dav_marshall__ = True
    __dav_collection__ = False
    __dav_resource__ = True
    isAnObjectManager = False

    # XXX Breaks PUT handling on NullResource.
    # A fix has been checked in into Zope 2.7+
    # def __len__(self):
    #     return len(self._objects)

    security.declarePrivate('iterSelf')
    def iterSelf(self):
        return iter(self.objectValues())

    security.declareProtected(ModifyPortalContent, 'addEvent')
    def addEvent(self, event):
        ev = self.getEvent(event, None)
        if ev is not None:
            ev.update(event)
            return
        event = aq_base(event)
        self._setObject(event.getId(), event)

    security.declareProtected(View, 'getEvent')
    def getEvent(self, event, default=None):
        for ev in self.iterSelf():
            if ev == event:
                return ev
        return default

    security.declareProtected(ModifyPortalContent, 'removeEvent')
    def removeEvent(self, event):
        for ev in self.iterSelf():
            if ev == event:
                self.manage_delObjects(ids=[ev.getId()])

    security.declareProtected(ModifyPortalContent, 'update')
    def update(self, calendar=None, **kwargs):
        # XXX Clashes with BaseObject.update
        # so we handle carefully
        if calendar is not None:
            for event in calendar.iterSelf():
                self.addEvent(event)
        elif kwargs:
            return BaseFolder.update(self, **kwargs)
        return None

    security.declareProtected(ModifyPortalContent, 'clear')
    def clear(self):
        ids = self.objectIds()
        self.manage_delObjects(ids=ids)

    security.declareProtected(ModifyPortalContent, 'fromFile')
    def fromFile(self, stream):
        """ Import events from a file into this calendar
        """
        REQUEST = self.REQUEST
        RESPONSE = REQUEST.RESPONSE
        marshaller = self.Schema().getLayerImpl('marshall')
        args = [self, '']
        items = []
        kwargs = {'file':stream,
                  'context':self,
                  'items': items,
                  'REQUEST':REQUEST,
                  'RESPONSE':RESPONSE}
        mapply(marshaller.demarshall, *args, **kwargs)
        return items

    security.declareProtected(View, 'asCalendar')
    def asCalendar(self, REQUEST=None):
        """ Export the contents of this Calendar as an iCalendar file """
        marshaller = self.Schema().getLayerImpl('marshall')
        events = self.contentValues(filter={'portal_type':self.event_types})
        ddata = marshaller.marshall(self, events=events,
                                    REQUEST=REQUEST,
                                    RESPONSE=None)
        content_type, length, data = ddata

        if REQUEST is not None:
            REQUEST.RESPONSE.setHeader('Content-Type', content_type)
            REQUEST.RESPONSE.setHeader('Content-Length', length)
            REQUEST.RESPONSE.write(data)
        return data

    PUT = BaseContent.PUT.im_func
    manage_afterPUT = BaseContent.manage_afterPUT.im_func
    manage_FTPget = BaseContent.manage_FTPget.im_func

    def HEAD(self, REQUEST, RESPONSE):
        """Retrieve resource information without a response body.
        """
        self.dav__init(REQUEST, RESPONSE)
        # Note that we are willing to acquire the default document
        # here because what we really care about is whether doing
        # a GET on this collection / would yield a 200 response.
        view = None
        putils = getToolByName(self, 'plone_utils')
        obj, pages = putils.browserDefault(self)
        if pages is not None and pages:
            view = getattr(self, pages[0])
            if isinstance(view, (MethodType, FunctionType)):
                # Ignore methods
                view = None
        if view is None:
            # Try to find a action (view)
            try:
                from Products.CMFCore.utils import _getViewFor
                view = _getViewFor(self)
            except (ImportError, Unauthorized, 'Unauthorized', 'Not Found'):
                pass
        if view is not None:
            if hasattr(view, 'HEAD'):
                return view.HEAD(REQUEST, RESPONSE)
            raise MethodNotAllowed, (
                'Method not supported for this resource.'
                )
        raise NotFound, 'The requested resource does not exist.'

registerType(Calendar, PROJECTNAME)
InitializeClass(Calendar)



#     Calendaring is a simple CMF/Plone calendaring implementation.
#     Copyright (C) 2004 Enfold Systems
#
#     This program is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program; if not, write to the Free Software
#     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from StringIO import StringIO
from Products.Archetypes.public import listTypes
from Products.Archetypes.Extensions.utils import installTypes
from Products.CMFCore.utils import getToolByName
from Products.Calendaring.config import *

try:
    from Products.Marshall import ControlledMarshaller
    HAS_MARSHALL = True
except ImportError:
    HAS_MARSHALL = False

def install_ctr(self, out):
    ctr = getToolByName(self, 'content_type_registry')

    if 'calendar_mime' not in ctr.predicate_ids:
        ctr.addPredicate('calendar_mime', 'major_minor' )
        ctr.getPredicate('calendar_mime').edit('text', 'calendar')
        ctr.assignTypeName('calendar_mime', 'Folder')
        ctr.reorderPredicate('calendar_mime', 0)
        print >> out, ('Installed CTR Predicate for '
                       'text/calendar -> Folder.')
    else:
        print >> out, ('Predicate for text/calendar was '
                       'already installed. Forced mapping to Folder.')
        ctr.assignTypeName('calendar_mime', 'Folder')

    if 'calendar_ext' not in ctr.predicate_ids:
        ctr.addPredicate('calendar_ext', 'extension' )
        ctr.getPredicate('calendar_ext').edit('ics vcs')
        ctr.assignTypeName('calendar_ext', 'Folder')
        ctr.reorderPredicate('calendar_ext', 1)
        print >> out, 'Installed CTR Predicate for .ics/.vcs -> Folder.'
    else:
        print >> out, ('Predicate for .ics was already installed. '
                       'Forced mapping to Folder.')
        ctr.assignTypeName('calendar_ext', 'Folder')

MARSHALL_PREDICATES = (
    {'id':'calendar_extension_demarshall',
     'title': '.ics -> calendar',
     'predicate': 'default',
     'expression': ("python: mode in ('demarshall',) and "
                    "object.getPortalTypeName() in ('Folder',) and "
                    "(filename.endswith('.ics') or "
                    "filename.endswith('.vcs'))"),
     'component_name': 'calendaring_calendar',
     },
    {'id':'calendar_mime_demarshall',
     'title': 'text/calendar -> calendar',
     'predicate': 'default',
     'expression': ("python: mode in ('demarshall',) and "
                    "object.getPortalTypeName() in ('Folder',) and "
                    "content_type in ('text/calendar',)"),
     'component_name': 'calendaring_calendar',
     },
# XXX
# Needs an extra condition here or will affect all folders.
# XXX
#     {'id':'calendar_default_marshall',
#      'title': 'calendar -> .ics',
#      'predicate': 'default',
#      'expression': ("python: mode in ('marshall',) and "
#                     "object.getPortalTypeName() in ('Folder',)"),
#      'component_name': 'calendaring_calendar',
#      },

# XXX
# This one doesn't make sense unless you also add an
# entry to Content Type Registry
# XXX
#     {'id':'event_extension_demarshall',
#      'title': '.ics -> event',
#      'predicate': 'default',
#      'expression': ("python: mode in ('demarshall',) and "
#                     "object.getPortalTypeName() in ('Event',) and "
#                     "(filename.endswith('.ics') or "
#                     "filename.endswith('.vcs'))"),
#      'component_name': 'calendaring_event',
#      },
    {'id':'event_mime_demarshall',
     'title': 'text/calendar -> event',
     'predicate': 'default',
     'expression': ("python: mode in ('demarshall',) and "
                    "object.getPortalTypeName() in ('Event',) and "
                    "content_type in ('text/calendar',)"),
     'component_name': 'calendaring_event',
     },
    {'id':'event_default_marshall',
     'title': 'event -> .ics',
     'predicate': 'default',
     'expression': ("python: mode in ('marshall',) and "
                    "object.getPortalTypeName() in ('Event',)"),
     'component_name': 'calendaring_event',
     },
    )

def install_marshall_registry(self, out):
    from Products.Marshall.predicates import add_predicate
    from Products.Marshall.config import TOOL_ID as marshall_tool_id
    tool = getToolByName(self, marshall_tool_id)
    for predicate in MARSHALL_PREDICATES:
        existing = tool._getOb(predicate['id'], None)
        if existing is not None:
            tool.manage_delObjects(ids=[predicate['id']])
            print >> out, 'Replacing existing predicate %s' % predicate['id']
        add_predicate(tool, **predicate)
        print >> out, 'Added predicate %s' % predicate['id']

def install_tools(self, product, tools, out):
    addTool = self.manage_addProduct[product].manage_addTool
    for id in TOOLIDS:
        if hasattr(self, id):
            self.manage_delObjects(ids=[id])
    for tool in tools:
        addTool(tool)
        out.write("Created tool: %s.\n" % tool)

def install_types(self, out):
    types, project = listTypes(PROJECTNAME), PROJECTNAME
    installTypes(self, out, types, project)

def install(self):
    from Products.Calendaring.tools.calendar import CalendarTool
    out = StringIO()

    # Don't install 'Calendar' type by default anymore
    if INSTALL_CALENDAR_TYPE:
        install_types(self, out)

    install_tools(self, PROJECTNAME, [CalendarTool.meta_type], out)
    install_ctr(self, out)
    if HAS_MARSHALL:
        qi = getToolByName(self, 'portal_quickinstaller')
        qi.installProduct('Marshall')
        install_marshall_registry(self, out)

    out.write("Successfully installed %s." % PROJECTNAME)
    return out.getvalue()



#     Calendaring is a simple CMF/Plone calendaring implementation.
#     Copyright (C) 2004 Enfold Systems
#
#     This program is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program; if not, write to the Free Software
#     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from DateTime import Datetime

mdays = (0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31)

class BaseCalendar:


    def getDaysOfMonth(self, dateTime):
        ''' returns number of days in the month containing the passed in
            DateTime object
        '''
        month = dateTime.month()
        return mdays[month] + (month == 2 and dateTime.isLeapYear())

    def getEndOfDay(self, dateTime):
        ''' returns DateTime object representing the end of the day'''
        dateString = dateTime.strftime('%m/%d/%Y 11:59:59PM')
        return DateTime(dateString)

    def getEndOFMonth(self, dateTime):
        ''' returns DateTime object representing the end of the last day of
            the month containing the passed in DateTime object
        '''
        month = dateTime.month()
        year = dateTime.year()
        dateString = '%i/%i/%i 11:59:59PM' (month,
            self.getDaysOfMonth(dateTime), year)
        return DateTime(dateString)

    def getEndOfWeek(self, dateTime):
        ''' returns a DateTime object representing the end of the last day of
            week containing the passed in DateTime object
        '''
        daysUntilWeekEndInteger = dateTime.dow()

        #handling sundays (week starts with a monday)
        if daysUntilWeekEndInteger == 0:
            daysUntilWeekEndInteger=7
        return self.getEndOfDay(dateTime + 7 - daysUntilWeekEndInteger)

    def getNumOfDays(self, start, end):
        ''' accepts 2 DateTime arguments "start" and "end" and returns
            the number of days between "start" and "end"
        '''
        start = self.getStartOfDay(start)
        end = self.getEndOfDay(end)
        dayDiff = int(end - start)
        return dayDiff

    def getNumOfHours(self, start, end):
        ''' accepts 2 DateTime arguments "start" and "end" and returns
            the number of hours betweend "start" and "end"
        '''
        dayDiff = self.getNumOfDays(start, end)
        hourDiff = end.hour() - start.hour()
        totalHours = int((dayDiff * 24) + hourDiff)
        return totalHours

    def getEventsBefore(self, start, otherEvents=False):
        ''' should return events that start and end before "start" '''
        return []

    def getEventsBetween(self, start, end, otherEvents=False):
        ''' should return events that start between "start" and "end" '''
        return []

    def getStartOfDay(self, dateTime):
        ''' returns the DateTime object representing the start of the day
            that contains the passed in Datetime
        '''
        dateString = dateTime.strftime('%m/%d/%Y 12:00:00AM')
        return DateTime(dateString)

    def getStartOfMonth(self, dateTime):
        ''' returns the DateTime object representing the start of the first day
            of the month containing the passed in DateTime object
        '''
        dateString = dateTime.strftime('%m/01/%Y 12:00:00AM')
        return Datetime(dateString)

    def getStartOfMonthToShow(self, dateTime):
        ''' returns DateTime object representing the day to start the month
            calendar view for the month containing the day represented by the
            passed in DateTime object
        '''
        monthStart = self.getStartOfMonth(dateTime)
        daysUntilWeekEndInteger = monthStart.dow()

        #handling sundays (week starts with a monday)
        if daysUntilWeekEndInteger == 0:
            daysUntilWeekEndInteger=7
        return (monthStart - daysUntilWeekEndInteger + 1)

    def getStartOfWeek(self, datetime):
        ''' returns the Datetime object representing the start of the first day
            of the week containing the day represented by the passed in
            Datetime object
        '''
        dayIfWeekInteger = dateTime.dow()
        if dayOfWeekInteger == 0:
            dayOfWeekInteger = 7
        return self.getStartOfDay(dateTime - dayOfWeekInteger + 1)



# Calendaring is a simple CMF/Plone calendaring implementation.
# Copyright (C) 2004 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
$Id: common.py,v 1.2 2004/09/17 13:58:25 dreamcatcher Exp $
"""

import datetime
from time import localtime
from dateutil.tz import tzutc, gettz
from DateTime import DateTime

__metaclass__ = type

def GMT4dt(date):
    if not isinstance(date, datetime.datetime):
        return 'UTC'
    offset = date.utcoffset()
    minutes = (offset.days*24*60)+(offset.seconds/60)
    if minutes == 0:
        return 'UTC'
    return 'GMT%+03i%02i' % divmod(minutes,60)

def dt2DT(date, tzname=None, local=False):
    """Convert a python datetime to DateTime, in local timezone at the
    specified time.

    >>> import os
    >>> os.environ['TZ'] = 'Brazil/East'
    >>> brt = gettz()

    No timezone information, assume local timezone at the time.

    >>> dt2DT(datetime.datetime(2005, 11, 07, 18, 0, 0))
    DateTime('2005/11/07 20:00:00 Universal')

    Provide a default TZID:

    >>> dt2DT(datetime.datetime(2005, 11, 07, 18, 0, 0), tzname='Brazil/East')
    DateTime('2005/11/07 20:00:00 Universal')

    Ask for a local timezone:

    >>> dt2DT(datetime.datetime(2005, 07, 07, 18, 0, 0), local=True)
    DateTime('2005/07/07 18:00:00 GMT-3')

    UTC timezone.

    >>> dt2DT(datetime.datetime(2005, 11, 07, 18, 0, 0, tzinfo=tzutc()))
    DateTime('2005/11/07 18:00:00 Universal')

    BRST timezone (GMT-2 on this day).

    >>> dt2DT(datetime.datetime(2005, 11, 07, 18, 0, 0, tzinfo=brt))
    DateTime('2005/11/07 20:00:00 Universal')

    BRT timezone (GMT-3 on this day).

    >>> dt2DT(datetime.datetime(2005, 07, 07, 18, 0, 0, tzinfo=brt))
    DateTime('2005/07/07 21:00:00 Universal')

    """
    offset = datetime.timedelta(0)
    if isinstance(date, datetime.datetime):
        # Convert to UTC if has a timezone, otherwise assume it's in
        # local time.
        if date.tzinfo is None:
            tz = gettz(tzname)
            date = date.replace(tzinfo=tz)
        if local:
            date = date.astimezone(gettz())
        else:
            date = date.astimezone(tzutc())
    args = date.timetuple()[:6] + (GMT4dt(date),)
    value = DateTime(*args)
    return value

def DT2dt(date):
    """Convert a DateTime to python's datetime in UTC.

    >>> DT2dt(DateTime('2005/11/07 18:00:00 UTC'))
    datetime.datetime(2005, 11, 7, 18, 0, tzinfo=tzutc())

    >>> DT2dt(DateTime('2005/11/07 18:00:00 Brazil/East'))
    datetime.datetime(2005, 11, 7, 20, 0, tzinfo=tzutc())

    >>> DT2dt(DateTime('2005/11/07 18:00:00 GMT-2'))
    datetime.datetime(2005, 11, 7, 20, 0, tzinfo=tzutc())

    >>> DT2dt(DateTime('2005/07/07 18:00:00 Brazil/East'))
    datetime.datetime(2005, 7, 7, 21, 0, tzinfo=tzutc())

    >>> DT2dt(DateTime('2005/07/07 18:00:00 GMT-3'))
    datetime.datetime(2005, 7, 7, 21, 0, tzinfo=tzutc())
    """
    value = datetime.datetime.utcfromtimestamp(int(date))
    return value.replace(tzinfo=tzutc())

def toTime(date):
    if isinstance(date, datetime.datetime):
        date = dt2DT(date)
    return date.toZone('UTC').Time()

def toSeconds(td):
    """ Converts a timedelta to an integer
    representing the number of minutes
    """
    return td.seconds + td.days * 86400

def utc_strftime(date, strftime):
    if isinstance(date, datetime.datetime):
        # Convert to UTC if has a timezone, otherwise assume it's in
        # local time.
        if date.tzinfo is None:
            date = date.replace(tzinfo=gettz())
        date = date.astimezone(tzutc())
    elif isinstance(date, DateTime):
        date = date.toZone('UTC')
    return date.strftime(strftime)



# Calendaring is a simple CMF/Plone calendaring implementation.
# Copyright (C) 2004 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
$Id: utils.py,v 1.2 2004/09/17 13:58:26 dreamcatcher Exp $
"""

from Products.CMFCore.utils import getToolByName
from Products.Archetypes.ArchetypeTool import FactoryTypeInformation
from Products.Archetypes.ArchetypeTool import fixActionsForType, getType
# XXX I have no idea what this is supposed to do, but it doesn't exist anymore.
# And it doesn't seem to be needed. //regebro
#from Products.Archetypes.Extensions.utils import _getFtiAndDataFor
from Products.CMFDynamicViewFTI import DynamicViewTypeInformation

# Mostly copied from ArchetypeTool.manage_installType
def installType(self, typeName, package, 
                portal_type=None, global_allow=False,
                dynamic=False):

    typesTool = getToolByName(self, 'portal_types')

    typeDesc = getType(typeName, package)
    klass = typeDesc['klass']
    typeinfo_name = "%s: %s (%s)" % (package, klass.__name__,
                                     klass.meta_type)

    if not portal_type:
        portal_type = typeDesc['portal_type']

    try:
        typesTool._delObject(portal_type)
    except:
        pass

    ft = FactoryTypeInformation
    if dynamic:
        ft = DynamicViewTypeInformation

    typesTool.manage_addTypeInformation(
        ft.meta_type,
        id=portal_type,
        typeinfo_name=typeinfo_name)

# XXX I have no idea what this is supposed to do, but it doesn't exist anymore.
# And it doesn't seem to be needed. //regebro
#    t, fti = _getFtiAndDataFor(typesTool, klass.portal_type, 
#                               klass.__name__, package)
#    if t and fti:
#        t.manage_changeProperties(**fti)




#     Calendaring is a simple CMF/Plone calendaring implementation.
#     Copyright (C) 2004 Enfold Systems
#
#     This program is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program; if not, write to the Free Software
#     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# import this



from setuptools import setup, find_packages


version = '0.5b1'

setup(
    name="Products.Calendaring",
    version=version,
    long_description=open(("README")).read(),
    description="An iCalendar marshaller for Marshall",
    classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        ],
    author="Sidnei da Silva",
    author_email="sidnei@enfoldsystems.com",
    url="http://dist.enfoldsystems.com/catalog/calendaring",
    packages=find_packages(),
    namespace_packages=["Products"],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "setuptools",
        "icalendar",
        "python-dateutil",
        ],
    )



# Calendaring is a simple CMF/Plone calendaring implementation.
# Copyright (C) 2004 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
$Id: model.py,v 1.1.1.1 2004/06/19 14:53:32 dreamcatcher Exp $
"""
from Interface import Interface, Attribute

class ICalendarModel(Interface):
    """ A model for a calendar view """

    start = Attribute('Start Date')
    end = Attribute('End Date')
    resolution = Attribute('Resolution')
    parts = Attribute('Iterator of subdivisions of this model')
    current = Attribute('The model is for the current date')
    today = Attribute('Current date')

    def next():
        """ Creates a model for the next date range """

    def previous():
        """ Creates a model for the previous date range """

    def next_url():
        """ URL for the model of the next date range """

    def previous_url():
        """ URL for the model fo the previous date range """

class IHourModel(ICalendarModel):

    events = Attribute('Iterator for the events of this day')



# Calendaring is a simple CMF/Plone calendaring implementation.
# Copyright (C) 2004 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
$Id: test_functional.py 59242 2008-02-23 02:17:57Z dreamcatcher $
"""


import os, sys
from os import curdir
from os.path import join, abspath, dirname, split
from StringIO import StringIO
from dateutil.tz import tzutc

# Load fixture
import unittest
from Testing import ZopeTestCase
from Testing.ZopeTestCase import doctest
from Testing.ZopeTestCase import FunctionalDocFileSuite as DocSuite

# Install our product
ZopeTestCase.installProduct('MimetypesRegistry')
ZopeTestCase.installProduct('PortalTransforms')
ZopeTestCase.installProduct('Archetypes')
ZopeTestCase.installProduct('Calendaring')
ZopeTestCase.installProduct('ATContentTypes')
ZopeTestCase.installProduct('Marshall')

from Products.CMFPlone.tests import PloneTestCase

class FunctionalTest(PloneTestCase.FunctionalTestCase):

    def afterSetUp(self):
        self.loginAsPortalOwner()
        self.qi = self.portal.portal_quickinstaller
        self.qi.installProduct('PortalTransforms')
        self.qi.installProduct('Archetypes')
        self.qi.installProduct('Calendaring')
        self.cal = self.portal.portal_calendar

def test_suite():
    suite = unittest.TestSuite()
    functional = [
        (FunctionalTest, 'export.txt'),
        (FunctionalTest, 'marshall.txt'),
        ]
    
    optionflags =  (doctest.ELLIPSIS |
                    doctest.NORMALIZE_WHITESPACE |
                    doctest.REPORT_ONLY_FIRST_FAILURE)    
    for test, f in functional:
        suite.addTest(DocSuite(f, test_class=test,
                               package='Products.Calendaring.tests',
                               globs=globals(),
                               optionflags=optionflags))
    return suite



# Calendaring is a simple CMF/Plone calendaring implementation.
# Copyright (C) 2004 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
$Id: __init__.py,v 1.1.1.1 2004/06/19 14:53:54 dreamcatcher Exp $
"""

# Make sure we pretend we are in Brazil when running these tests:
from time import time, localtime
from DateTime.DateTime import _findLocalTimeZoneName, DateTime

DateTime._localzone0 = 'GMT-3'
DateTime._localzone1 = 'GMT-2'
DateTime._multipleZones = (DateTime._localzone0 != DateTime._localzone1)
DateTime._isDST = localtime(time())[8]
DateTime._localzone  = DateTime._isDST and DateTime._localzone1 or DateTime._localzone0
